function [upper_bound, false_pos_count, set_negs, thresholds_of_UB] = computeInitialUpperBound(data, neg_scores)

%    Copyright (C) 2015  Davide Modolo
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options.
%
%    Contact: d.modolo@sms.ed.ac.uk

false_pos_count = zeros(size(data.t4p));

% Loop over the exemplars and calculate how many false positives we get
% for setting the threshold of exemplar 'i' to 't' so that positive 'p'
% is retrieved. 't' is the highest threshold (in the discrete space) 
% necessary to score each 'p' positive
for i = 1:size(data.t4p, 2)
    one_eesvm_data.fp_thresh = data.fp_thresh(:, i);
    false_pos_count(:, i) = one_eesvm_data.fp_thresh(data.t4p(:, i));
end

% Pick one threshold per positive
[~, i_min_f] = min(false_pos_count, [], 2);

% Count the negatives
set_negs = [];
for i = 1:size(data.t4p, 1)    
    set_negs = [set_negs; find(neg_scores(:, i_min_f(i)) > ...
        data.candidate_thresholds(data.t4p(i, i_min_f(i)), i_min_f(i)))];    
end

% Eliminate negatives "shared" by many E-SVMs
set_negs = unique(set_negs);
upper_bound = size(set_negs, 1);
 
% Loop over all E-SVMs
% For each E-SVM look up what threshold was used in the solution that lead
% to the upper bound
thresholds_of_UB = zeros(size(data.fp_thresh, 2), 1);
for i = 1:size(data.fp_thresh, 2)

    e = find(i_min_f == i);
    if ~isempty(e)
        tr_indx = data.t4p(e, i);        
        thresholds_of_UB(i) = data.candidate_thresholds(max(tr_indx), i);
    else
        thresholds_of_UB(i) = data.candidate_thresholds(i, 1);
    end
end     